@@ -1,5 +1,3 @@
-#!/usr/bin/env python
-#
 # Thin convenience wrapper for BioPython's SeqIO code
 # Copyright (C) 2012 Kai Blin <kai.blin@biotech.uni-tuebingen.de>
 #
@@ -15,16 +13,20 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#
+'''Wrappers for Bio.SeqIO'''
 
-from Bio import SeqIO
+import sys
 from os import path
 try:
     from cStringIO import StringIO
 except ImportError:
-    from StringIO import StringIO
+    from Bio._py3k import StringIO
+from Bio import SeqIO
+from Bio._py3k import basestring
+
 
 def _get_seqtype_from_ext(handle):
+    '''Predict the filetype from a handle's name'''
     if isinstance(handle, basestring):
         name = handle
     elif hasattr(handle, 'filename'):
@@ -33,13 +35,22 @@ def _get_seqtype_from_ext(handle):
         name = handle.name
     else:
         raise ValueError("Unknown datatype for handle!")
+
+    modifier = ''
     dummy, ext = path.splitext(name.lower())
+    if ext == ".gz":
+        modifier = 'gz-'
+        dummy, ext = path.splitext(dummy)
+
+    if not ext:
+        ext = "." + dummy
+
     if ext in (".gbk", ".gb", ".genbank", ".gbff"):
-        return "genbank"
+        return modifier + "genbank"
     elif ext in (".embl", ".emb"):
-        return "embl"
+        return modifier + "embl"
     elif ext in (".fa", ".fasta", ".fna", ".faa", ".fas"):
-        return "fasta"
+        return modifier + "fasta"
     else:
         raise ValueError("Unknown file format '%s'." % ext)
 
@@ -73,6 +84,36 @@ def _guess_seqtype_from_file(handle):
     raise ValueError("Failed to guess format for input")
 
 
+def _unzip_handle(handle):
+    """Transparently unzip the file handle"""
+    if isinstance(handle, basestring):
+        handle = _gzip_open_filename(handle)
+    else:
+        handle = _gzip_open_handle(handle)
+    return handle
+
+
+def _gzip_open_filename(handle):
+    """Hide Python 2 vs. 3 differences in gzip.open()"""
+    import gzip
+    if sys.version_info[0] > 2:
+        handle = gzip.open(handle, mode='rt', encoding="UTF-8")
+    else:
+        handle = gzip.open(handle)
+    return handle
+
+
+def _gzip_open_handle(handle):
+    """"Hide Python 2 vs. 3 differences in gzip.GzipFile()"""
+    import gzip
+    if sys.version_info[0] > 2:
+        import io
+        handle = io.TextIOWrapper(gzip.GzipFile(fileobj=handle), encoding="UTF-8")
+    else:
+        handle = gzip.GzipFile(fileobj=handle)
+    return handle
+
+
 def sanity_check_insdcio(handle, id_marker, fake_id_line):
     """Sanity check for insdcio style files"""
     found_id = False
@@ -140,8 +181,17 @@ def sanity_check_fasta(handle):
     return new_handle
 
 
-def parse(handle, robust=False):
-    seqtype = _get_seqtype_from_ext(handle)
+def parse(handle, seqtype=None, robust=False):
+    '''Wrap SeqIO.parse'''
+    if seqtype is None:
+        seqtype = _get_seqtype_from_ext(handle)
+
+    if seqtype.startswith('gz-'):
+        handle = _unzip_handle(handle)
+        seqtype = seqtype[3:]
+
+    # False positive from pylint, both handles are fileobj-like
+    # pylint: disable=redefined-variable-type
     if robust:
         if seqtype == "embl":
             handle = sanity_check_embl(handle)
@@ -149,12 +199,22 @@ def parse(handle, robust=False):
             handle = sanity_check_genbank(handle)
         elif seqtype == "fasta":
             handle = sanity_check_fasta(handle)
+    # pylint: enable=redefined-variable-type
 
     return SeqIO.parse(handle, seqtype)
 
 
-def read(handle, robust=False):
-    seqtype = _get_seqtype_from_ext(handle)
+def read(handle, seqtype=None, robust=False):
+    '''Wrap SeqIO.read'''
+    if seqtype is None:
+        seqtype = _get_seqtype_from_ext(handle)
+
+    if seqtype.startswith('gz-'):
+        handle = _unzip_handle(handle)
+        seqtype = seqtype[3:]
+
+    # False positive from pylint, both handles are fileobj-like
+    # pylint: disable=redefined-variable-type
     if robust:
         if seqtype == "embl":
             handle = sanity_check_embl(handle)
@@ -162,8 +222,11 @@ def read(handle, robust=False):
             handle = sanity_check_genbank(handle)
         elif seqtype == "fasta":
             handle = sanity_check_fasta(handle)
+    # pylint: enable=redefined-variable-type
+
     return SeqIO.read(handle, seqtype)
 
 
 def write(*args, **kwargs):
+    '''Just pass through SeqIO.write'''
     SeqIO.write(*args, **kwargs)
